/**--------------------------------------------------------------------------
 **
 **		NEC PC-8001 Emulator "N80" for Win32/DirectX
 **		--------------------------------------------
 **		Copyright (c) 1993-2010 H.Kanemoto
 **
 **		Assay N80 Performance
 **
 **		Rev		Date		Descriptions
 **		-----	----------	-----------------------------------------------
 **		1.00	1996/06/27	porting DOS version on WinG
 **		1.20	1997/08/10	porting WinG version on Win32/DirectX
 **		1.30	1998/03/12	
 **		1.43	2007/09/01	XPΉ
 **		1.44	2009/12/15	Vista/7Ή
 **		1.51	2010/01/27	DirectSound OnlyɕύX
 **------------------------------------------------------------------------*/
#include "n80_assay.h"
#include <stdio.h>
#include <stdarg.h>
#include <crtdbg.h>
/**--------------------------------------------------------------------------
 **		RXgN^^fXgN^
 **------------------------------------------------------------------------*/
n80_assay::n80_assay()
{
	m_mutex = ::CreateMutex(NULL, FALSE, "ASSAYMUTEX");
	init();
}

n80_assay::~n80_assay()
{
	showResult();
	dispose();
}
/**--------------------------------------------------------------------------
 **		^j֐
 **------------------------------------------------------------------------*/
void
n80_assay::init()
{
#if defined(_TEST)
	// Logt@CǋL[hI[v
	m_file = ::CreateFile(
		"N80.log",
		GENERIC_READ | GENERIC_WRITE | SYNCHRONIZE,
		FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL,
		OPEN_ALWAYS,
		FILE_ATTRIBUTE_NORMAL | FILE_FLAG_WRITE_THROUGH,
		(HANDLE) NULL);
	if (INVALID_HANDLE_VALUE != m_file) {
		::SetFilePointer(m_file, 0, NULL, FILE_END);
	}
#endif
#if defined(_TEST) || defined(_DEBUG)
	m_log_open_failed = 0;

	m_z80_hires_base_ticks = 0;
	m_z80_hires_rates = 0;
	m_z80_adjust_skips = 0;
	m_z80_loops = 0;
	m_z80_loops_waited = 0;
	m_z80_sleep_cnt = 0;

	m_tick_delay = 0;
	m_feed_total = 0;

	m_feed_over  = 0;
	m_feed_rack  = 0;
	m_feed_log_ticks = 0;
	m_feed_pys_ticks = 0;

	m_feed1_over  = 0;
	m_feed1_rack  = 0;
	m_feed1_log_ticks = 0;
	m_feed1_pys_ticks = 0;
#endif
}

void
n80_assay::dispose()
{
	if (m_mutex != NULL) {
		::CloseHandle(m_mutex);
		m_mutex = NULL;
	}
#if defined(_TEST)
	if (INVALID_HANDLE_VALUE != m_file) {
		::CloseHandle(m_file);
		m_file = INVALID_HANDLE_VALUE;
	}
#endif
}
/**--------------------------------------------------------------------------
 **		Oo͊֐
 **------------------------------------------------------------------------*/
void
n80_assay::outputLog(LPCTSTR fmt,...)
{
	TCHAR	msg[1024];
	::ZeroMemory(msg, sizeof(msg));
	va_list arg;
	va_start(arg, fmt);
	vsprintf_s(msg, sizeof(msg), fmt, arg);
	va_end(arg);

#if defined(_TEST)
	if (INVALID_HANDLE_VALUE == m_file) {
		++m_log_open_failed;
	} else {
		TCHAR	buff[1024];
		SYSTEMTIME	st;
		DWORD wsiz = 0;

		if (::WaitForSingleObject(m_mutex, INFINITE) != WAIT_OBJECT_0)
			return;
		::GetLocalTime(&st);
		::ZeroMemory(buff, sizeof(buff));
		sprintf_s(buff, sizeof(buff),
			"%04d/%02d/%02d %02d:%02d:%02d\t%s\n",
			st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, msg);
		::WriteFile(m_file, buff, lstrlen(buff), &wsiz, NULL);
		::ReleaseMutex(m_mutex);
	}
#elif defined(_DEBUG)
	TCHAR	buff[1024];
	SYSTEMTIME	st;
	DWORD wsiz = 0;

	if (::WaitForSingleObject(m_mutex, INFINITE) != WAIT_OBJECT_0)
		return;
	::GetLocalTime(&st);
	::ZeroMemory(buff, sizeof(buff));
	sprintf_s(buff, sizeof(buff),
		">%04d/%02d/%02d %02d:%02d:%02d\t%s\n",
		st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond, msg);
	_RPT0(_CRT_WARN, buff);
	::ReleaseMutex(m_mutex);

#endif
}

void
n80_assay::outputLastError()
{
#if defined(_TEST) || defined(_DEBUG)
	LPTSTR lpBuffer;
	DWORD le = ::GetLastError();
	::FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		NULL,
		le,
		LANG_USER_DEFAULT,//MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT),
		(LPTSTR)&lpBuffer,
		0,
		NULL);
	outputLog(TEXT("LastError: %s"), lpBuffer);
	::LocalFree(lpBuffer);
#endif
}
/**--------------------------------------------------------------------------
 **		Z80sx
 **------------------------------------------------------------------------*/
void
n80_assay::setZ80ticks(LONGLONG ticks)
{
#if defined(_TEST) || defined(_DEBUG)
	// Z80x[vx^C}[Tick
	m_z80_hires_base_ticks = ticks;
#endif
}

void
n80_assay::measureZ80Clock(int loops, LONGLONG ticks)
{
#if defined(_TEST) || defined(_DEBUG)
	++m_z80_loops;
	if (loops > 0)
		++m_z80_loops_waited;
	m_z80_hires_rates += (((double) ticks) / ((double) m_z80_hires_base_ticks));
#endif
}

void
n80_assay::countZ80Sleep()
{
#if defined(_TEST) || defined(_DEBUG)
	++m_z80_sleep_cnt;
#endif
}
/**--------------------------------------------------------------------------
 **		BeeptԊu
 **------------------------------------------------------------------------*/
void
n80_assay::beginTakeInterval()
{
#if defined(_TEST) || defined(_DEBUG)
	// tԊu菀
	LARGE_INTEGER freq, percnt;
	::QueryPerformanceFrequency(&freq);
	::QueryPerformanceCounter(&percnt);
	m_per_freq = freq.QuadPart;
	m_feed_percounter = percnt.QuadPart;
	m_feed_intervals = 0.0;
#endif
}

void
n80_assay::changeTakeInterval(DWORD z80clk)
{
#if defined(_TEST) || defined(_DEBUG)
	m_tick_delay = (LONGLONG) (z80clk / 5);	// T-states per 200ms(0.2s)
#endif
}

void
n80_assay::takeFeedIntervals()
{
#if defined(_TEST) || defined(_DEBUG)
	// tԊu
	LARGE_INTEGER percnt;
	::QueryPerformanceCounter(&percnt);
	LONGLONG interval = percnt.QuadPart - m_feed_percounter;
	m_feed_percounter = percnt.QuadPart;
	double intsec = ((double) interval) / ((double) m_per_freq);
	m_feed_intervals += intsec;
#endif
}
void
n80_assay::takeFeedTicks(LONGLONG beepticks, LONGLONG pcgticks)
{
#if defined(_TEST) || defined(_DEBUG)
	// t񐔗݌v
	++m_feed_total;

	// TvTicksWv
	m_feed_log_ticks += m_tick_delay;
	m_feed_pys_ticks += beepticks;
	m_feed1_log_ticks += m_tick_delay;
	m_feed1_pys_ticks += pcgticks;

	// XgbNTicks1ms𒴉߂Ăꍇ
	if (beepticks > m_tick_delay) {
		++m_feed_over;
	}
	else if (beepticks < m_tick_delay) {
		++m_feed_rack;
	}
	if (pcgticks > m_tick_delay) {
		++m_feed1_over;
	}
	else if (pcgticks < m_tick_delay) {
		++m_feed1_rack;
	}
#endif
}
/**--------------------------------------------------------------------------
 **		ʏo
 **------------------------------------------------------------------------*/
void
n80_assay::showResult()
{
#if defined(_TEST) || defined(_DEBUG)
	// DirectSoundgpɂ̂ݏWv\
	outputLog(TEXT("Z80WaitF %lf()"), ((double) m_z80_loops_waited) / ((double) m_z80_loops) * 100);
	outputLog(TEXT("Z80WaitTickF %lf()"), m_z80_hires_rates / ((double) m_z80_loops) * 100);
	outputLog(TEXT("Z80 Sleep 񐔁F%d()"), m_z80_sleep_cnt);
	outputLog(TEXT("Z80 SleepF %lf()"), m_z80_sleep_cnt / ((double) m_z80_loops) * 100);
	if (m_tick_delay > 0) {
		outputLog(TEXT("t񐔁F%lf()"), m_feed_intervals);
		outputLog(TEXT("ωtԁF%lf(sec)"), m_feed_intervals / ((double) m_feed_total));
		outputLog(TEXT("Tv[F %lf()"), ((double) m_feed_pys_ticks) / ((double) m_feed_log_ticks) * 100);
		outputLog(TEXT("TvߊF %lf()"), ((double) m_feed_over) / ((double) m_feed_total) * 100);
		outputLog(TEXT("TvsF %lf()"), ((double) m_feed_rack) / ((double) m_feed_total) * 100);
		outputLog(TEXT("Tv[F %lf()"), ((double) m_feed1_pys_ticks) / ((double) m_feed1_log_ticks) * 100);
		outputLog(TEXT("TvߊF %lf()"), ((double) m_feed1_over) / ((double) m_feed_total) * 100);
		outputLog(TEXT("TvsF %lf()"), ((double) m_feed1_rack) / ((double) m_feed_total) * 100);
	}
#endif
}
