/**--------------------------------------------------------------------------
 **
 **		NEC PC-8001 Emulator "N80" for Win32/DirectX
 **		--------------------------------------------
 **		Copyright (c) 1993-2010 H.Kanemoto
 **
 **		Load/Save Profile
 **
 **		Rev		Date		Descriptions
 **		-----	----------	-----------------------------------------------
 **		1.00	1996/06/27	porting DOS version on WinG
 **		1.20	1997/08/10	porting WinG version on Win32/DirectX
 **		1.30	1998/03/12	
 **		1.43	2007/09/01	XPΉ
 **		1.44	2009/12/15	Vista/7Ή
 **		1.51	2010/01/27	DirectSound OnlyɕύX
 **------------------------------------------------------------------------*/
#include "n80_form.h"
#include "n80_directx.h"
#include "../z80/z80_thread.h"
#include "../lib/icj_stream.h"
#include <stdio.h>
#include <string.h>
#include <crtdbg.h>
/**--------------------------------------------------------------------------
 **		O[oϐ
 **------------------------------------------------------------------------*/
extern n80_directx	*dx;
extern z80_thread	*th;
/**--------------------------------------------------------------------------
 **		W[ϐ
 **------------------------------------------------------------------------*/
static LPCTSTR	l_section = {TEXT("Default")};
static LPCTSTR	l_key1    = {TEXT("z80_no_wait")};
static LPCTSTR	l_key2    = {TEXT("dma_gain")};
static LPCTSTR	l_key3    = {TEXT("hard_volume")};
static LPCTSTR	l_key4    = {TEXT("soft_volume")};
static LPCTSTR	l_key6    = {TEXT("use_videomem")};
static LPCTSTR	l_key8    = {TEXT("dma_load")};
static LPCTSTR	l_key9    = {TEXT("sync_vblank")};
static LPCTSTR	l_key10   = {TEXT("tenkeyless")};
static LPCTSTR	l_key11   = {TEXT("screensize")};
static LPCTSTR	l_key12   = {TEXT("diff_draw")};
static LPCTSTR	l_key13   = {TEXT("interlace")};

#define N80_DEF_LOAD	100		// DMA load
#define N80_DEF_GAIN	20		// DMA gain
#define N80_DEF_HWVOL	(-4)	// hardware volume
#define N80_DEF_SWVOL	(1)		// software volume
#define N80_DEF_TKLESS	1		// ten key less
#define N80_DEF_NOWAIT	0		// z80 no-wait false
#define N80_DEF_SYNCVB	0		// wait vertical blank
#define N80_DEF_SCRNSIZ	0		// screen size (0:640x400, 1:720x480)
#define N80_DEF_VIDEO	1		// use video ram
#define N80_DEF_DEMAND	0		// draw text on demand
#define N80_DEF_INTLCE	0		// interlace like
/**--------------------------------------------------------------------------
 **		INIt@C
 **------------------------------------------------------------------------*/
BOOL
n80_form::initProfile()
{
	icj_fstream	fs;

	::GetCurrentDirectory(sizeof (mIniPath), mIniPath);
	lstrcat(mIniPath, TEXT("\\N80.INI"));

	if (!fs.open(mIniPath)) {
		if (!fs.create(mIniPath))
			return  FALSE;
		fs.close();
		loadProfile(NULL);
		saveProfile(NULL);
	} else {
		fs.close();
		loadProfile(NULL);
	}
	return  TRUE;
}
/**--------------------------------------------------------------------------
 **		INIt@Cǂݍ
 **------------------------------------------------------------------------*/
void
n80_form::loadProfile(LPCTSTR sect)
{
	INT  val;

	if (sect == NULL)
		sect = l_section;

	val = ::GetPrivateProfileInt(sect, l_key1, N80_DEF_NOWAIT, mIniPath);
	th->setZ80NoWait(val == 1 ? TRUE : FALSE);

	val = ::GetPrivateProfileInt(sect, l_key2, N80_DEF_GAIN, mIniPath);
	th->setDMALoadGain(val);

	val = ::GetPrivateProfileInt(sect, l_key3, N80_DEF_HWVOL, mIniPath);
	m_hard_volume = val;

	val = ::GetPrivateProfileInt(sect, l_key4, N80_DEF_SWVOL, mIniPath);
	m_soft_volume = val;

	val = ::GetPrivateProfileInt(sect, l_key6, N80_DEF_VIDEO, mIniPath);
	mUseVideo = TRUE;// val == 1 ? TRUE : FALSE;

	val = ::GetPrivateProfileInt(sect, l_key8, N80_DEF_LOAD, mIniPath);
	n80device.dmac.dma_penalty_base = (DWORD) val;

	val = ::GetPrivateProfileInt(sect, l_key9, N80_DEF_SYNCVB, mIniPath);
	dx->setSyncVBlank(val == 1 ? TRUE : FALSE);

	val = ::GetPrivateProfileInt(sect, l_key10, N80_DEF_TKLESS, mIniPath);
	mTenKeyless = val == 1 ? TRUE : FALSE;

	val = ::GetPrivateProfileInt(sect, l_key11, N80_DEF_SCRNSIZ, mIniPath);
	dx->setScreenSize(val == 0 ? 0 : 1);

	val = ::GetPrivateProfileInt(sect, l_key12, N80_DEF_DEMAND, mIniPath);
	dx->setTxtOnDemand(val == 1 ? TRUE : FALSE);

	val = ::GetPrivateProfileInt(sect, l_key13, N80_DEF_INTLCE, mIniPath);
	n80device.param.interlace = val == 1 ? TRUE : FALSE;
}
/**--------------------------------------------------------------------------
 **		INIt@Co
 **------------------------------------------------------------------------*/
void
n80_form::saveProfile(LPCTSTR sect)
{
	char  num[64];

	if (sect == NULL)
		sect = l_section;

	sprintf(num, "%d", th->getZ80NoWait() ? 1 : 0);
	::WritePrivateProfileString(sect, l_key1, num, mIniPath);

	sprintf(num, "%u", th->getDMALoadGain());
	::WritePrivateProfileString(sect, l_key2, num, mIniPath);

	sprintf(num, "%d", m_hard_volume);
	::WritePrivateProfileString(sect, l_key3, num, mIniPath);

	sprintf(num, "%d", m_soft_volume);
	::WritePrivateProfileString(sect, l_key4, num, mIniPath);

	sprintf(num, "%u", mUseVideo ? 1 : 0);
	::WritePrivateProfileString(sect, l_key6, num, mIniPath);

	sprintf(num, "%u", n80device.dmac.dma_penalty_base);
	::WritePrivateProfileString(sect, l_key8, num, mIniPath);

	sprintf(num, "%u", dx->getSyncVBlank() ? 1 : 0);
	::WritePrivateProfileString(sect, l_key9, num, mIniPath);

	sprintf(num, "%u", mTenKeyless ? 1 : 0);
	::WritePrivateProfileString(sect, l_key10, num, mIniPath);

	sprintf(num, "%d", dx->getScreenSize() == 0 ? 0 : 1);
	::WritePrivateProfileString(sect, l_key11, num, mIniPath);

	sprintf(num, "%u", dx->getTxtOnDemand() ? 1 : 0);
	::WritePrivateProfileString(sect, l_key12, num, mIniPath);

	sprintf(num, "%u", n80device.param.interlace ? 1 : 0);
	::WritePrivateProfileString(sect, l_key13, num, mIniPath);
}
