/**==========================================================================

	PC-8001 Device Emulator for Win32 Ver 1.10
	Copyright (C)1993-2010 H.Kanemoto

	This is a free software. You may copy or distribute
	it without any permissions. But there are NO WARRANTY
	of use. Entire risk of use is with you.

	External assembler functions for IBM-PC with VC++ 5.0
	last modified  Apr,4,1998.

;==========================================================================*/
#include "pc-8001.h"
#include <stdio.h>
#ifdef _MSC_VER
#include <windows.h>
#endif
#ifdef __cplusplus
extern "C"
{
#endif
#define export
/**--------------------------------------------------------------------------
 **		ASM Modules
 **------------------------------------------------------------------------*/
static void _n80_reset_vrtc(void) {
	if (n80device.crtc._line_mode == 1) {		// 25lines
		n80device.vrtc.hsync_dsp_state = 179;	// horizontal display interval (44.70us)
		n80device.vrtc.hsync_all_state = 250;	// horizontal blanking interval(17.88us/62.58us)
		n80device.vrtc.vsync_bck_state = 13994;	// vertical blanking interval  (3.504ms)
	} else {									// 20lines
		n80device.vrtc.hsync_dsp_state = 179;	// horizontal display interval (44.70us)
		n80device.vrtc.hsync_all_state = 250;	// horizontal blanking interval(17.88us/62.58us)
		n80device.vrtc.vsync_bck_state = 14996;	// vertical blanking interval  (3.755ms)
	}
	n80device.vrtc.vsync_line  = 0;
	n80device.vrtc.hsync_state = 0;
	n80device.vrtc.hsync_state = 0;
	n80device.vrtc._vrtc_flag  = 0;
	n80device.vrtc._hsync_start_flg = 1;
	n80device.vrtc._dma_remain_chars = 0;
}

static int _n80_blanking_vrtc(DWORD T) {
	int result = _VRTC_DISP;
	BYTE rasidx;
	if (n80device.vrtc.vsync_line < 200) {
		//hsync start
		if (n80device.vrtc._hsync_start_flg == 1) {
			n80device.vrtc._hsync_start_flg = 0;
			//raster index of line
			rasidx = n80device.vrtc.vsync_line % n80device.crtc._raster_per_char;
			if (rasidx == 0) {
				// dma underrun error occured
				if (n80device.vrtc._dma_remain_chars > 0) {
					// ignore in emulator
					fprintf(stderr, "DMA under-run occured!!!\n");
				}
				// DMA transfer start (CPU WAIT) if dma ch.2 is active or crts start display
				if (n80device.crtc._burst_mode == 1) {
				// dma bulk transfer request (burst mode)
					n80device.dmac._penarty = (n80device.crtc._crtc_line_bytes / 2) * 11;
					n80device.vrtc._dma_remain_chars = 0;
				} else {	
					// dma char by char transfer request
					n80device.vrtc._dma_remain_chars = n80device.crtc._crtc_line_bytes;
					n80device.dmac._penarty = 0;
				}
			}
		}
		// dma transfer
		if ((n80device.crtc._text_visible & 0x06) == 0) {
			if (n80device.vrtc._dma_remain_chars > 0) {
				--n80device.vrtc._dma_remain_chars;
				n80device.dmac._penarty += (n80device.vrtc._dma_remain_chars & 0x01) == 1 ? 5 : 6;
			}
		} else {	// not-visible (crtc/dma stopped)
			n80device.vrtc._dma_remain_chars = 0;
			n80device.dmac._penarty = 0;
		}
		// vertical display interval
		n80device.vrtc.hsync_state += T;
		if (n80device.vrtc.hsync_state < n80device.vrtc.hsync_dsp_state) {
			// horizontal display interval
			;
		} else if (n80device.vrtc.hsync_state < n80device.vrtc.hsync_all_state) {
			// horizontal blanking interval
			;
		} else {	// end of 1-line display
			if (++n80device.vrtc.vsync_line >= 200) {
				// vertical blanking interval start
				n80device.vrtc.vsync_state = (n80device.vrtc.hsync_state - n80device.vrtc.hsync_all_state);
                n80device.vrtc.hsync_state = 0;
				n80device.vrtc._vrtc_flag  = 1;
				n80device.vrtc.edge();
				n80device.dmac._penarty = 0;
				// autoload DMAC ch3 to ch2 (VRAM transfer)
				if (n80device.dmac._autoload == 1) {
					n80device.dmac.ch[2]._addr = n80device.dmac.ch[3]._addr;
					n80device.dmac.ch[2]._cnt  = n80device.dmac.ch[3]._cnt;
					n80device.dmac._begin      = n80device.dmac.ch[2]._addr;
					n80device.dmac._end        = n80device.dmac._begin + n80device.dmac.ch[2]._cnt;
				}
				result = _VRTC_VBBGN;
			} else {
				n80device.vrtc.hsync_state -= n80device.vrtc.hsync_all_state;
				n80device.vrtc._hsync_start_flg = 1;
			}
		}
	} else {	// vertical blanking interval
		n80device.vrtc.vsync_state += T;
		if (n80device.vrtc.vsync_state >= n80device.vrtc.vsync_bck_state) {		// end of VBI
			n80device.vrtc.vsync_line   = 0;
			n80device.vrtc.hsync_state  = n80device.vrtc.vsync_state - n80device.vrtc.vsync_bck_state;
            n80device.vrtc.vsync_state  = 0;
			n80device.vrtc._vrtc_flag   = 0;
			result = _VRTC_VBEND;
		}
	}
	return result;
}
	
static void _n80_renew_vrtc(void)
{
	DWORD cur_nible, att_nible;
	++n80device.vrtc._vrtc_blink;
	cur_nible = n80device.vrtc._vrtc_blink % n80device.cursor._cur_blink_cycl;
	att_nible = n80device.vrtc._vrtc_blink % n80device.cursor._att_blink_cycl;
	n80device.cursor._blinking = cur_nible < n80device.cursor._cur_blink_disp ? 1 : 0;
	n80device.cursor._attr_blinking = att_nible < n80device.cursor._att_blink_disp ? 1 : 0;
}

static BYTE _n80_poll_vrtc(BYTE i40)
{
	if (n80device.vrtc._vrtc_flag == 1)
		return i40 |= 0x20;
	else
		return i40 &= 0xdf;
}
	
export void _n80_init_vrtc(void) {
	n80device.vrtc.reset = _n80_reset_vrtc;
	n80device.vrtc.blanking = _n80_blanking_vrtc;
	n80device.vrtc.edge = _n80_renew_vrtc;
	n80device.vrtc.poll = _n80_poll_vrtc;
	_n80_reset_vrtc();
}
	
#ifdef __cplusplus
}	// extern "C"
#endif
